//+------------------------------------------------------------------+
//|                                                      ProjectName |
//|                                      Copyright 2012, CompanyName |
//|                                       http://www.companyname.net |
//+------------------------------------------------------------------+
#property copyright ""
#property link      ""

#include <MovingAverages.mqh>

#property indicator_chart_window

#property indicator_buffers   9
#property indicator_plots     1

#property strict
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
enum ENUM_PRICE
  {
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average,    // Average (high+low+open+close)/4
   pr_medianb,    // Average median body (open+close)/2
   pr_tbiased,    // Trend biased price
   pr_haclose,    // Heiken ashi close
   pr_haopen,     // Heiken ashi open
   pr_hahigh,     // Heiken ashi high
   pr_halow,      // Heiken ashi low
   pr_hamedian,   // Heiken ashi median
   pr_hatypical,  // Heiken ashi typical
   pr_haweighted, // Heiken ashi weighted
   pr_haaverage,  // Heiken ashi average
   pr_hamedianb,  // Heiken ashi median body
   pr_hatbiased   // Heiken ashi trend biased price
  };
//---
input uint        CalcPeriod  =  15;              // Calculation period
input ENUM_PRICE  CalcPrice   =  pr_close;        // Price
input color       ColorUp     =  clrDeepSkyBlue;  // Color for up trend
input color       ColorDown   =  clrSandyBrown;   // Color for down trend
input uint        LinesWidth  =  2;               // Lines width for lines
input uint        BodyWidth   =  2;               // Body width for bars

//---
ENUM_CHART_MODE _chart_mode;

//---
double OpenBuffer[];
double HighBuffer[];
double LowBuffer[];
double CloseBuffer[];
double ColorBuffer[];

double PriceBuffer[];
double MaBuffer[];

double HaOpenBuffer[];
double HaCloseBuffer[];
//+------------------------------------------------------------------+
//|   OnInit                                                         |
//+------------------------------------------------------------------+
int OnInit()
  {

//Print("Init");

   ArraySetAsSeries(OpenBuffer,true);
   ArraySetAsSeries(HighBuffer,true);
   ArraySetAsSeries(LowBuffer,true);
   ArraySetAsSeries(CloseBuffer,true);
   ArraySetAsSeries(ColorBuffer,true);

   ArraySetAsSeries(PriceBuffer,true);
   ArraySetAsSeries(MaBuffer,true);

   ArraySetAsSeries(HaOpenBuffer,true);
   ArraySetAsSeries(HaCloseBuffer,true);
//---
   _chart_mode=(ENUM_CHART_MODE)ChartGetInteger(0,CHART_MODE);
   switch(_chart_mode)
     {
      case CHART_LINE:
         SetIndexBuffer(0,CloseBuffer,INDICATOR_DATA);
         SetIndexBuffer(1,ColorBuffer,INDICATOR_COLOR_INDEX);

         SetIndexBuffer(2,PriceBuffer,INDICATOR_CALCULATIONS);
         SetIndexBuffer(3,MaBuffer,INDICATOR_CALCULATIONS);

         SetIndexBuffer(4,HaOpenBuffer,INDICATOR_CALCULATIONS);
         SetIndexBuffer(5,HaCloseBuffer,INDICATOR_CALCULATIONS);

         PlotIndexSetInteger(0,PLOT_DRAW_TYPE,DRAW_COLOR_LINE);
         PlotIndexSetInteger(0,PLOT_LINE_WIDTH,LinesWidth);
         PlotIndexSetString(0,PLOT_LABEL,"Line");
         break;
         //---
      case CHART_BARS:
         SetIndexBuffer(0,HighBuffer,INDICATOR_DATA);
         SetIndexBuffer(1,LowBuffer,INDICATOR_DATA);
         SetIndexBuffer(2,ColorBuffer,INDICATOR_COLOR_INDEX);

         SetIndexBuffer(3,PriceBuffer,INDICATOR_CALCULATIONS);
         SetIndexBuffer(4,MaBuffer,INDICATOR_CALCULATIONS);

         SetIndexBuffer(5,HaOpenBuffer,INDICATOR_CALCULATIONS);
         SetIndexBuffer(6,HaCloseBuffer,INDICATOR_CALCULATIONS);

         PlotIndexSetInteger(0,PLOT_DRAW_TYPE,DRAW_COLOR_HISTOGRAM2);
         PlotIndexSetInteger(0,PLOT_LINE_WIDTH,BodyWidth);
         PlotIndexSetString(0,PLOT_LABEL,"High;Low");
         break;
         //---
      case CHART_CANDLES:
         SetIndexBuffer(0,OpenBuffer,INDICATOR_DATA);
         SetIndexBuffer(1,HighBuffer,INDICATOR_DATA);
         SetIndexBuffer(2,LowBuffer,INDICATOR_DATA);
         SetIndexBuffer(3,CloseBuffer,INDICATOR_DATA);
         SetIndexBuffer(4,ColorBuffer,INDICATOR_COLOR_INDEX);

         SetIndexBuffer(5,PriceBuffer,INDICATOR_CALCULATIONS);
         SetIndexBuffer(6,MaBuffer,INDICATOR_CALCULATIONS);

         SetIndexBuffer(7,HaOpenBuffer,INDICATOR_CALCULATIONS);
         SetIndexBuffer(8,HaCloseBuffer,INDICATOR_CALCULATIONS);

         PlotIndexSetInteger(0,PLOT_DRAW_TYPE,DRAW_COLOR_CANDLES);
         PlotIndexSetString(0,PLOT_LABEL,"Open;High;Low;Close");
         break;
     }
//---
   PlotIndexSetInteger(0,PLOT_COLOR_INDEXES,2);
   PlotIndexSetInteger(0,PLOT_LINE_COLOR,0,ColorUp);
   PlotIndexSetInteger(0,PLOT_LINE_COLOR,1,ColorDown);
//---
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
//---   
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,const long &lparam,const double &dparam,const string &sparam)
  {
   ENUM_CHART_MODE chart_mode=(ENUM_CHART_MODE)ChartGetInteger(0,CHART_MODE);
   if(chart_mode!=_chart_mode)
     {
      _chart_mode=chart_mode;
      OnInit();
      ChartSetSymbolPeriod(ChartID(),_Symbol,_Period);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
// Print("OnCalculate");
   if(rates_total<(int)CalcPeriod+1)
      return(0);

   ArraySetAsSeries(open,true);
   ArraySetAsSeries(high,true);
   ArraySetAsSeries(low,true);
   ArraySetAsSeries(close,true);
   ArraySetAsSeries(time,true);
//---
   int limit;
   if(prev_calculated==0)
     {
      limit=rates_total-(int)CalcPeriod-1;
      ArrayInitialize(OpenBuffer,EMPTY_VALUE);
      ArrayInitialize(HighBuffer,EMPTY_VALUE);
      ArrayInitialize(LowBuffer,EMPTY_VALUE);
      ArrayInitialize(CloseBuffer,EMPTY_VALUE);
      ArrayInitialize(ColorBuffer,0);
      ArrayInitialize(PriceBuffer,0);
      ArrayInitialize(MaBuffer,0);
      ArrayInitialize(HaCloseBuffer,0);
      ArrayInitialize(HaOpenBuffer,0);
     }
   else
      limit=rates_total-prev_calculated;

//--- price   
   getPrice(rates_total,prev_calculated,CalcPrice,open,high,low,close,PriceBuffer);

//--- ma
   ExponentialMAOnBuffer(rates_total,prev_calculated,0,CalcPeriod,PriceBuffer,MaBuffer);

//--- main cycle
   for(int i=limit; i>=0 && !_StopFlag; i--)
     {

      if(_chart_mode==CHART_LINE)
        {
         CloseBuffer[i]=MaBuffer[i];
        }

      if(_chart_mode==CHART_BARS)
        {
         HighBuffer[i]=high[i];
         LowBuffer[i]=low[i];
        }

      if(_chart_mode==CHART_CANDLES)
        {
         OpenBuffer[i]=open[i];
         HighBuffer[i]=high[i];
         LowBuffer[i]=low[i];
         CloseBuffer[i]=close[i];
        }

      PriceBuffer[i]=NormalizeDouble(PriceBuffer[i],8);
      MaBuffer[i]=NormalizeDouble(MaBuffer[i],8);

      if(PriceBuffer[i]>MaBuffer[i])
        {
         ColorBuffer[i]=0;
        }
      if(PriceBuffer[i]<MaBuffer[i])
        {
         ColorBuffer[i]=1;
        }
     }

   return(rates_total);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void getPrice(const int rates_total,
              const int prev_calculated,
              const ENUM_PRICE price_type,
              const double &open[],
              const double &high[],
              const double &low[],
              const double &close[],
              double &price[])
  {

   int limit;
   if(prev_calculated==0)
     {
      limit=rates_total-1;
     }
   else
      limit=rates_total-prev_calculated;
//---
   if(price_type>=pr_haclose)
     {
      for(int i=limit; i>=0 && !_StopFlag; i--)
        {
         double haOpen=0.0;
         if(i==rates_total-1)
            haOpen=(open[i]+close[i])/2.0;
         else
            haOpen=(HaOpenBuffer[i+1]+HaCloseBuffer[i+1])/2.0;

         double haClose=(open[i]+high[i]+low[i]+close[i])/4.0;
         double haHigh=fmax(high[i],fmax(haOpen,haClose));
         double haLow=fmin(low[i],fmin(haOpen,haClose));

         HaOpenBuffer[i]=haOpen;
         HaCloseBuffer[i]=haClose;

         //---
         switch(price_type)
           {
            case pr_haclose:     price[i]=haClose; break;
            case pr_haopen:      price[i]=haOpen; break;
            case pr_hahigh:      price[i]=haHigh; break;
            case pr_halow:       price[i]=haLow; break;
            case pr_hamedian:    price[i]=(haHigh+haLow)/2.0; break;
            case pr_hamedianb:   price[i]=(haOpen+haClose)/2.0; break;
            case pr_hatypical:   price[i]=(haHigh+haLow+haClose)/3.0;break;
            case pr_haweighted:  price[i]=(haHigh+haLow+haClose+haClose)/4.0; break;
            case pr_haaverage:   price[i]=(haHigh+haLow+haClose+haOpen)/4.0; break;
            case pr_hatbiased:
               if(haClose>haOpen)
               price[i]=(haHigh+haClose)/2.0;
               else
                  price[i]=(haLow+haClose)/2.0;
               break;
           }
        }
     }
   else
     {
      for(int i=limit; i>=0 && !_StopFlag; i--)
        {
         switch(price_type)
           {
            case pr_close:     price[i]=close[i]; break;
            case pr_open:      price[i]=open[i]; break;
            case pr_high:      price[i]=high[i]; break;
            case pr_low:       price[i]=low[i]; break;
            case pr_median:    price[i]=(high[i]+low[i])/2.0; break;
            case pr_medianb:   price[i]=(open[i]+close[i])/2.0; break;
            case pr_typical:   price[i]=(high[i]+low[i]+close[i])/3.0; break;
            case pr_weighted:  price[i]=(high[i]+low[i]+close[i]+close[i])/4.0; break;
            case pr_average:   price[i]=(high[i]+low[i]+close[i]+open[i])/4.0; break;
            case pr_tbiased:
               if(close[i]>open[i])
               price[i]=(high[i]+close[i])/2.0;
               else
                  price[i]=(low[i]+close[i])/2.0;
               break;
           }
        }
     }
  }
//+------------------------------------------------------------------+
